use crate::{structs::json::Tile, tui::text::add_text, utils::maps::{get_current_map, get_map_events}};

pub fn move_player(x: i8, y: i8, dry_run: bool) -> bool {
    let mut all_coords = crate::PLAYER_COORDS.lock().unwrap();
    let player_coords = *all_coords.get(get_current_map().unwrap().id.as_str()).unwrap();
    // let mut new_coords = (player_coords.0 + x, player_coords.1 + y);
    let new_coords = (
        player_coords.0.wrapping_add_signed(x as i8),
        player_coords.1.wrapping_add_signed(y as i8),
    );

    let map = get_current_map().unwrap();

    let width = map.width as u8;
    let height = map.height as u8;
    
    // check if new coordinates are within bounds
    if new_coords.0 >= width || new_coords.1 >= height {
        return false; // Out of bounds, do not move
    }

    // check if new coordinates are onto walkable tile
    let tile_name = map.tilemap.get(new_coords.1 as usize)
        .and_then(|row| row.get(new_coords.0 as usize))
        .and_then(|tile| Some(tile.as_str()))
        .unwrap_or_default();

    let walkable = map.tiles.get(tile_name)
        .and_then(|tile| Some(tile.walkable))
        .unwrap_or(false);

    // check if player is allowed to enter onto this tile
    let events = get_map_events(get_current_map().unwrap().id.as_str()).unwrap();

    let evt = events.iter().find(|e| e.evt_type == "lock" && e.location[0] == new_coords.0 && e.location[1] == new_coords.1);
    if evt.is_some() {
        let evt = evt.unwrap();
        
        let required_item = evt.command_id.clone();
        let player_items = crate::PLAYER_INVENTORY.lock().unwrap().clone();
        if !player_items.contains(&required_item) {
            if !dry_run { add_text(evt.description.as_ref().unwrap()); }
            return false; // Player does not have required item to enter this tile
        }
    }

    if dry_run { return walkable; }

    if !walkable {
        add_text(&format!("! Cannot move into {} tile", tile_name));
        return false; // Not a walkable tile, do not move
    }

    all_coords.insert(map.id, new_coords);
    *crate::PLAYER_STAMINA.lock().unwrap() -= 1;

    // crate::PLAYER_COORDS.lock().unwrap().0 = new_coords.0;
    // crate::PLAYER_COORDS.lock().unwrap().1 = new_coords.1;

    add_text(&format!("You moved into {} tile", tile_name));

    return true;
}

pub fn get_current_tile() -> Option<Tile> {
    let map = get_current_map().unwrap();
    let player_coords = crate::PLAYER_COORDS.lock().unwrap().clone();
    let map_coords = *player_coords.get(map.id.as_str()).unwrap();

    let tile_id = map.tilemap
        .get(map_coords.1 as usize)
        .and_then(|row| row.get(map_coords.0 as usize))
        .and_then(|tile| Some(tile.as_str()))
        .unwrap_or_default();

    let tile = map.tiles.get(tile_id)
        .cloned()
        .unwrap();

    Some(tile)
}