use colored::Colorize;
use terminal_size::{Width, Height, terminal_size};

use crate::{parser::nlp::get_available_commands, tui::{map::draw_tile, text::refresh_text}, utils::{maps::get_current_map, os::move_cursor}};

pub fn draw_interface() {
    move_cursor(0, 0);

    let Some((Width(w), Height(h))) = terminal_size()
    else {
        eprintln!("Could not determine terminal size.");
        return;
    };

    let horizontal_char = "─";
    let vertical_char = "│";
    // let corner_char = "+";
    let perp_down_char = "┬";
    let perp_up_char = "┴";
    let perp_left_char = "├";
    let perp_right_char = "┤";
    let corner_down_left_char = "┌";
    let corner_down_right_char = "┐";
    let corner_up_left_char = "└";
    let corner_up_right_char = "┘";

    let horizontal_line = horizontal_char.repeat(usize::from(w - 2));

    print!("{}", String::from(corner_down_left_char) + &horizontal_line + corner_down_right_char);
    for i in 0..(h - 2) {
        if i == 0 {
            print!("{}", String::from(vertical_char) + &" ".repeat(usize::from(w - 2)) + vertical_char);
        } else if i == 1 {
            print!("{}", String::from(perp_left_char) + &horizontal_char.repeat(16) + perp_down_char + &horizontal_char.repeat(usize::from(w - 19 - 25)) + perp_down_char + &horizontal_char.repeat(24) + perp_right_char);
        } else if i == h - 9 || i == 10 {
            print!("{}", String::from(perp_left_char) + &horizontal_char.repeat(16) + perp_right_char + &" ".repeat(usize::from(w - 19 - 25)) + vertical_char + &" ".repeat(24) + vertical_char);
        } else if i < h - 4 {
            print!("{}", String::from(vertical_char) + &" ".repeat(16) + vertical_char + &" ".repeat(usize::from(w - 19 - 25)) + vertical_char + &" ".repeat(24) + vertical_char);
        } else if i == h - 4 {
            print!("{}", String::from(perp_left_char) + &horizontal_char.repeat(16) + perp_up_char + &horizontal_char.repeat(usize::from(w - 19 - 25)) + perp_up_char + &horizontal_char.repeat(24) + perp_right_char);
        } else {
            print!("{}", String::from(vertical_char) + &" ".repeat(usize::from(w - 2)) + vertical_char);
        }
    }
    print!("{}", String::from(corner_up_left_char) + &horizontal_line + corner_up_right_char);

    draw_player_data();

    move_cursor(w - 21, 3);
    print!("{}", "POSSIBLE ACTIONS".bold().underline());

    move_cursor(1, 1);
    print!("Player: {} | Map: {}", crate::PLAYER_NAME.lock().unwrap().bold().underline(), get_current_map().unwrap().name.bold().underline());

    draw_tile();
    get_available_commands();
    
    refresh_text();
}

pub fn clear_input() {
    let Some((Width(width), Height(height))) = terminal_size()
    else {
        eprintln!("Could not determine terminal size.");
        return;
    };

    move_cursor(2, height - 2);
    
    print!("{:width$}", "", width = usize::from(width - 4));
    move_cursor(2, height - 2);
}

pub fn draw_player_data() {
    let Some((Width(_), Height(height))) = terminal_size()
    else {
        eprintln!("Could not determine terminal size.");
        return;
    };
    let y = height - 7;

    let hp = u16::try_from(*crate::PLAYER_HEALTH.lock().unwrap()).unwrap();
    let sp = u16::try_from(*crate::PLAYER_STAMINA.lock().unwrap()).unwrap();

    let max_hp = 100;
    let max_sp = 50;

    move_cursor(1, y);
    print!("HP: {}/{}", hp, max_hp);
    move_cursor(1, y + 1);
    let hp_blocks_count = ((hp as f64) / 100.0 * 16.0).ceil() as u8;
    let mut hp_blocks = String::from("");
    for _ in 0..hp_blocks_count {
        hp_blocks.push('█');
    }
    for _ in hp_blocks_count..16 {
        hp_blocks.push('_');
    }
    print!("{}", hp_blocks.red());

    move_cursor(1, y + 2);
    print!("SP: {}/{}", sp, max_sp);
    move_cursor(1, y + 3);
    let mp_blocks_count = ((sp as f64) / 50.0 * 16.0).ceil() as u8;
    let mut mp_blocks = String::from("");
    for _ in 0..mp_blocks_count {
        mp_blocks.push('█');
    }
    for _ in mp_blocks_count..16 {
        mp_blocks.push('_');
    }
    print!("{}", mp_blocks.yellow());
}
