use colored::Colorize;

use crate::{game::player::get_current_tile, utils::os::move_cursor};

pub fn draw_tile() {
    let tile = get_current_tile();

    let bitmap = tile.clone().unwrap().bitmap;

    for (y, row) in bitmap.iter().enumerate() {
        move_cursor(1, (y as u16) + 3);
        for (_, pixel) in row.iter().enumerate() {
            let colors = tile.clone().unwrap().icon.unwrap().color;

            print!("{}", pixel.on_truecolor(colors[0], colors[1], colors[2]));
        }
    }
}

pub fn draw_map() {
    clear_map();
    let map = crate::utils::maps::get_current_map().unwrap().clone();
    let player_coords = crate::PLAYER_COORDS.lock().unwrap().clone();
    let map_coords = *player_coords.get(map.id.as_str()).unwrap();

    let starting_index = (
        if map.width > 16 {
            if map_coords.0 > 8 {
                map_coords.0 as u16 - 8
            } else {
                0
            }
        } else { 0 },
        if map.height > 8 {
            if map_coords.1 > 4 {
                map_coords.1 as u16 - 4
            } else {
                0
            }
        } else { 0 }
    );

    for (y, row) in map.tilemap.iter().skip(starting_index.1 as usize).enumerate() {
        if y >= 8 { break }
        move_cursor(1, (y as u16) + 12);
        for (x, tile_id) in row.iter().skip(starting_index.0 as usize).enumerate() {
            if x + starting_index.0 as usize == map_coords.0 as usize && y + starting_index.1 as usize == map_coords.1 as usize {
                print!("{}", "X".bold().red().on_truecolor(228, 209, 178));
                continue;
            }
            let tile = map.tiles.get(tile_id.as_str()).unwrap();
            let icon = tile.icon.clone().unwrap();
            print!("{}", icon.char.truecolor(icon.color[0], icon.color[1], icon.color[2]).on_truecolor(228, 209, 178));
        }
    }
}

pub fn clear_map() {
    for y in 0..8 {
        move_cursor(1, 12 + y);
        print!("{}", " ".repeat(16));
    }
}
