use colored::{ColoredString, Colorize};
use terminal_size::{Width, Height, terminal_size};

use crate::utils::os::{move_cursor, set_cursor_visibility};

// pub fn clear_text() {
//     set_cursor_visibility(false);
//     // let Some((Width(width), Height(height))) = terminal_size()
//     // else {
//     //     eprintln!("Could not determine terminal size.");
//     //     return;
//     // };

//     // let text_area_width: u16 = width - 4 - 16 - 24;

//     move_cursor(18, 1);
//     // print!("{}", "x".repeat(usize::from(text_area_width)));
//     *crate::LINES_OF_TEXT.lock().unwrap() = [].to_vec();
// }

fn color_text(text: &str) -> ColoredString {
    if text.starts_with("#") {
        let color_code_raw = &text[2..8];
        let color_code = (
            u8::from_str_radix(&color_code_raw[0..2], 16).unwrap_or(255),
            u8::from_str_radix(&color_code_raw[2..4], 16).unwrap_or(255),
            u8::from_str_radix(&color_code_raw[4..6], 16).unwrap_or(255)
        );
        let text = &text[9..].truecolor(color_code.0, color_code.1, color_code.2);

        text.clone()
    } else {
        text.normal()
    }
}

pub fn add_text(text: &str) {
    set_cursor_visibility(false);
    let Some((Width(width), Height(height))) = terminal_size()
    else {
        eprintln!("Could not determine terminal size.");
        return;
    };
    let text_area_width: u16 = width - 4 - 16 - 24;

    let mut lines_of_text = crate::LINES_OF_TEXT.lock().unwrap();

    // move_cursor(18, (*lines_of_text + 1).try_into().unwrap_or(1));

    let wrapped_lines = wrap_text(text, text_area_width);
    for line in wrapped_lines {
        lines_of_text.push(line);
        print_text(lines_of_text.clone(), text_area_width, height - 6);
        // move_cursor(18, (*lines_of_text + 1).try_into().unwrap_or(1));
    }
}

pub fn wrap_text(text: &str, width: u16) -> Vec<String> {
    let mut wrapped_lines = Vec::new();
    let mut current_line = String::new();
    let words: Vec<&str> = text.split_whitespace().collect();

    for word in words {
        if current_line.len() + word.len() + 1 > usize::from(width) {
            wrapped_lines.push(current_line.trim().to_string());
            current_line.clear();
        }
        if !current_line.is_empty() {
            current_line.push(' ');
        }
        current_line.push_str(word);
    }

    if !current_line.is_empty() {
        wrapped_lines.push(current_line.trim().to_string());
    }

    wrapped_lines    
}

fn print_text(lines: Vec<String>, width: u16, height: u16) {
    set_cursor_visibility(false);

    let mut line_number = 3;

    if lines.len() > usize::from(height) {
        let start_index = lines.len() - usize::from(height);
        let end_index = lines.len();
        let visible_lines = &lines[start_index..end_index];
        for line in visible_lines.iter() {
            let text = color_text(&line);
            move_cursor(18, line_number);
            print!("{}{}", text, " ".repeat(usize::from(width) - line.len()));
            line_number += 1;
        }
    } else {
        for line in lines.iter() {
            let text = color_text(&line);
            move_cursor(18, line_number);
            print!("{}", text);
            line_number += 1;
        }
    }
}

pub fn refresh_text() {
    let Some((Width(width), Height(height))) = terminal_size()
    else {
        eprintln!("Could not determine terminal size.");
        return;
    };
    let text_area_width: u16 = width - 4 - 16 - 24;

    let lines_of_text = crate::LINES_OF_TEXT.lock().unwrap();

    print_text(lines_of_text.to_vec(), text_area_width, height);
}
